@@ -5,7 +5,6 @@ pub mod builder;
 pub mod chromosome;
 mod field;
 pub mod filters;
-pub mod format;
 pub mod genotypes;
 pub mod ids;
 pub mod info;
@@ -17,9 +16,8 @@ pub(crate) mod value;
 
 pub use self::{
     alternate_bases::AlternateBases, builder::Builder, chromosome::Chromosome, field::Field,
-    filters::Filters, format::Format, genotypes::Genotypes, ids::Ids, info::Info,
-    parser::ParseError, position::Position, quality_score::QualityScore,
-    reference_bases::ReferenceBases,
+    filters::Filters, genotypes::Genotypes, ids::Ids, info::Info, parser::ParseError,
+    position::Position, quality_score::QualityScore, reference_bases::ReferenceBases,
 };
 
 #[deprecated(
@@ -28,8 +26,16 @@ pub use self::{
 )]
 pub use self::genotypes::{genotype, Genotype};
 
+#[deprecated(
+    since = "0.11.0",
+    note = "Use `noodles_vcf::record::genotypes::Format` instead."
+)]
+pub use self::genotypes::Keys as Format;
+
 use std::{error, fmt, num, str::FromStr};
 
+use super::Header;
+
 pub(crate) const MISSING_FIELD: &str = ".";
 pub(crate) const FIELD_DELIMITER: char = '\t';
 
@@ -51,11 +57,15 @@ pub struct Record {
     quality_score: Option<QualityScore>,
     filters: Option<Filters>,
     info: Info,
-    format: Option<Format>,
     genotypes: Genotypes,
 }
 
 impl Record {
+    /// Parses a raw VCF record.
+    pub fn try_from_str(s: &str, header: &Header) -> Result<Self, ParseError> {
+        parser::parse(s, header)
+    }
+
     /// Returns a builder to create a record from each of its fields.
     ///
     /// # Examples
@@ -319,6 +329,8 @@ impl Record {
     ///
     /// # Examples
     ///
+    /// ```
+    /// use noodles_vcf::{self as vcf, record::{Filters, Position}};
     ///
     /// let record = vcf::Record::builder()
     ///     .set_chromosome("sq0".parse()?)
@@ -375,8 +387,8 @@ impl Record {
     ///     .build()?;
     ///
     /// let expected = Info::try_from(vec![
-    ///     Field::new(Key::SamplesWithDataCount, Value::Integer(3)),
-    ///     Field::new(Key::AlleleFrequencies, Value::FloatArray(vec![0.5])),
+    ///     Field::new(Key::SamplesWithDataCount, Some(Value::Integer(3))),
+    ///     Field::new(Key::AlleleFrequencies, Some(Value::FloatArray(vec![0.5]))),
     /// ])?;
     ///
     /// assert_eq!(record.info(), &expected);
@@ -404,12 +416,12 @@ impl Record {
     ///     .set_info("NS=3;AF=0.5".parse()?)
     ///     .build()?;
     ///
-    /// let dp = Field::new(Key::TotalDepth, Value::Integer(13));
+    /// let dp = Field::new(Key::TotalDepth, Some(Value::Integer(13)));
     /// record.info_mut().insert(dp.clone());
     ///
     /// let expected = Info::try_from(vec![
-    ///     Field::new(Key::SamplesWithDataCount, Value::Integer(3)),
-    ///     Field::new(Key::AlleleFrequencies, Value::FloatArray(vec![0.5])),
+    ///     Field::new(Key::SamplesWithDataCount, Some(Value::Integer(3))),
+    ///     Field::new(Key::AlleleFrequencies, Some(Value::FloatArray(vec![0.5]))),
     ///     dp,
     /// ])?;
     ///
@@ -427,31 +439,27 @@ impl Record {
     /// ```
     /// use noodles_vcf::{
     ///     self as vcf,
-    ///     record::{
-    ///         genotypes::{genotype::field::Key, Genotype},
-    ///         Format,
-    ///         Position,
-    ///     },
+    ///     record::{genotypes::{genotype::field::Key, Genotype, Keys}, Genotypes, Position},
     /// };
     ///
-    /// let format: Format = "GT:GQ".parse()?;
+    /// let keys = "GT:GQ".parse()?;
+    /// let genotypes = vec![Genotype::from_str_format("0|0:13", &keys)?];
     ///
     /// let record = vcf::Record::builder()
     ///     .set_chromosome("sq0".parse()?)
     ///     .set_position(Position::try_from(1)?)
     ///     .set_reference_bases("A".parse()?)
-    ///     .set_format(format.clone())
-    ///     .add_genotype(Genotype::from_str_format("0|0:13", &format)?)
+    ///     .set_genotypes(Genotypes::new(keys, genotypes))
     ///     .build()?;
     ///
-    /// assert_eq!(record.format(), Some(&Format::try_from(vec![
+    /// assert_eq!(record.format(), &Keys::try_from(vec![
     ///     Key::Genotype,
     ///     Key::ConditionalGenotypeQuality,
-    /// ])?));
+    /// ])?);
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
-    pub fn format(&self) -> Option<&Format> {
-        self.format.as_ref()
+    pub fn format(&self) -> &genotypes::Keys {
+        self.genotypes.keys()
     }
 
     /// Returns the genotypes of the record.
@@ -462,30 +470,29 @@ impl Record {
     /// use noodles_vcf::{
     ///     self as vcf,
     ///     record::{
-    ///         genotypes::{genotype::{field::{Key, Value}, Field}, Genotype},
-    ///         Format,
+    ///         genotypes::{genotype::{field::{Key, Value}, Field}, Genotype, Genotypes},
     ///         Position,
     ///     },
     /// };
     ///
-    /// let format: Format = "GT:GQ".parse()?;
+    /// let keys = "GT:GQ".parse()?;
+    /// let genotypes = Genotypes::new(
+    ///     keys,
+    ///     vec![Genotype::try_from(vec![
+    ///         Field::new(Key::Genotype, Some(Value::String(String::from("0|0")))),
+    ///         Field::new(Key::ConditionalGenotypeQuality, Some(Value::Integer(13))),
+    ///     ])?],
+    /// );
     ///
     /// let record = vcf::Record::builder()
     ///     .set_chromosome("sq0".parse()?)
     ///     .set_position(Position::try_from(1)?)
     ///     .set_reference_bases("A".parse()?)
-    ///     .set_format(format.clone())
-    ///     .add_genotype(Genotype::from_str_format("0|0:13", &format)?)
+    ///     .set_genotypes(genotypes.clone())
     ///     .build()?;
     ///
-    /// let expected = vec![
-    ///     Genotype::try_from(vec![
-    ///         Field::new(Key::Genotype, Some(Value::String(String::from("0|0")))),
-    ///         Field::new(Key::ConditionalGenotypeQuality, Some(Value::Integer(13))),
-    ///     ])?
-    /// ].into();
     ///
-    /// assert_eq!(record.genotypes(), &expected);
+    /// assert_eq!(record.genotypes(), &genotypes);
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
     pub fn genotypes(&self) -> &Genotypes {
@@ -572,7 +579,7 @@ impl Record {
 
         let end = if let Some(field) = self.info().get(&Key::EndPosition) {
             match field.value() {
-                Value::Integer(n) => *n,
+                Some(Value::Integer(n)) => *n,
                 _ => return Err(EndError::InvalidInfoEndPositionFieldValue),
             }
         } else {
@@ -615,8 +622,7 @@ impl fmt::Display for Record {
 
         write!(f, "\t{}", self.info())?;
 
-        if let Some(format) = self.format() {
-            write!(f, "\t{}", format)?;
+        if !self.genotypes().is_empty() {
             write!(f, "\t{}", self.genotypes())?;
         }
 
@@ -628,7 +634,7 @@ impl FromStr for Record {
     type Err = ParseError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        parser::parse(s)
+        Self::try_from_str(s, &Header::default())
     }
 }
 
@@ -644,7 +650,7 @@ mod tests {
             .set_reference_bases("A".parse()?)
             .set_info(Info::try_from(vec![info::Field::new(
                 info::field::Key::EndPosition,
-                info::field::Value::Flag,
+                Some(info::field::Value::Flag),
             )])?)
             .build()?;
 
@@ -681,14 +687,14 @@ mod tests {
     fn test_fmt_with_format() -> Result<(), Box<dyn std::error::Error>> {
         use genotypes::Genotype;
 
-        let format: Format = "GT:GQ".parse()?;
+        let keys: genotypes::Keys = "GT:GQ".parse()?;
+        let genotypes = vec![Genotype::from_str_format("0|0:13", &keys)?];
 
         let record = Record::builder()
             .set_chromosome("sq0".parse()?)
             .set_position(Position::try_from(1)?)
             .set_reference_bases("A".parse()?)
-            .set_format(format.clone())
-            .add_genotype(Genotype::from_str_format("0|0:13", &format)?)
+            .set_genotypes(Genotypes::new(keys, genotypes))
             .build()?;
 
         assert_eq!(
